#include "Demo.h"

#define BI_RGB 0L
typedef struct {
    U32 bfSize;
    U32 bfReserved;
    U32 bfOffBits;
    U32 biSize;
    S32 biWidth;
    S32 biHeight;
    U16 biPlanes;
    U16 biBitCount;
    U32 biCompression;
    U32 biSizeImage;
    S32 biXPelsPerMeter;
    S32 biYPelsPerMeter;
    U32 biClrUsed;
    U32 biClrImportant;
} BITMAPFILEHEADER;

typedef struct {
    U8 rgbBlue;
    U8 rgbGreen;
    U8 rgbRed;
    U8 rgbReserved;
} RGBQUAD;

#define EXTRACT_READ32(startaddr, offset) (U32)(startaddr[offset] + (U32)(startaddr[offset + 1] << 8) + (U32)(startaddr[offset + 2] << 16) + (U32)(startaddr[offset + 3] << 24))

#define SSD1351_CMD                       0 /**< command type */
#define SSD1351_DATA                      1 /**< data type */

#define SSD1351_CMD_SET_COLUMN_ADDRESS    0x15 /**< set column address command */
#define SSD1351_CMD_SET_ROW_ADDRESS       0x75 /**< set row address command */
#define SSD1351_CMD_WRITE_RAM             0x5C /**< write ram command */

static void lcd_cmd(U8 cmd)
{
    ssd1351_interface_cmd_data_gpio_write(SSD1351_CMD);
    ssd1351_interface_spi_write(&cmd, 1);
}
static void lcd_data(U8 data)
{
    ssd1351_interface_cmd_data_gpio_write(SSD1351_DATA);
    ssd1351_interface_spi_write(&data, 1);
}

uint8_t ssd1351_basic_draw_bmp(uint8_t left, uint8_t top, uint8_t right, uint8_t bottom, uint8_t *bmp)
{
    U32 datasize;
    U16 bpp;
    U32 w, h;
    U8 *bmpdata;
    U32 filesize;
    U32 offset;
    U32 bisize;
    int i, j;
    // parse header
    if ((bmp[0] != 0x42) || (bmp[1] != 0x4d)) // "BM"
    {
        debugprintf("Invalid BMP-ID\r\n");
        return 1;
    }
    filesize = EXTRACT_READ32(bmp, 2);
    offset   = EXTRACT_READ32(bmp, 10);
    bisize   = EXTRACT_READ32(bmp, 14);
    datasize = filesize - offset;
    bmpdata  = (U8 *)(offset + (U32)(bmp));
    bpp      = (U32)bmp[0x1c] + (U32)(bmp[0x1d] << 8);
    w        = EXTRACT_READ32(bmp, 0x12);
    h        = EXTRACT_READ32(bmp, 0x16);

    if ((bpp == 8) || (bpp == 8)) {

    } else if (bpp == 24) {
        // convert RGB565
        int ibpl;
        U16 rgb565;
        ibpl = ((w * 3) + 3) & 0xfffffffc;
        // flip
        bmpdata += ((h - 1) * ibpl);

        lcd_cmd(SSD1351_CMD_SET_COLUMN_ADDRESS);
        lcd_data(left);  // left
        lcd_data(right); // right
        lcd_cmd(SSD1351_CMD_SET_ROW_ADDRESS);
        lcd_data(top);    // top
        lcd_data(bottom); // bottom
        lcd_cmd(SSD1351_CMD_WRITE_RAM);

        ssd1351_interface_cmd_data_gpio_write(SSD1351_DATA);

        for (i = 0; i < h; i++) {
            for (j = 0; j < w; j++) {
                rgb565 = MAKE_RGB565(bmpdata[j * 3 + 2], bmpdata[j * 3 + 1], bmpdata[j * 3 + 0]);
                rgb565 = (rgb565 >> 8) | (rgb565 << 8);
                ssd1351_interface_spi_write(&rgb565, 2);
            }
            bmpdata -= ibpl; // 4byte align
        }
    }
    return 0; // sucess
}
