#include "demo.h"

INCBIN(testbmp, "bg.bmp");
INCBIN(bgwav, "adluna.wav");

 void evmboardinit();
 void amp_on(bool b);

#define CHECK_BIT(var, pos) (((var) >> (pos)) & 1)

JOYSTICK_TYPE get_joystick_type()
{
    static JOYSTICK_TYPE old_type = JOYSTICK_TYPE_NONE;
    JOYSTICK_TYPE new_type        = JOYSTICK_TYPE_NONE;
    int stat                      = *R_GPILEV(8) & 0x3e;

    if (stat == 0x3e)
        new_type = JOYSTICK_TYPE_NONE;

    if (!CHECK_BIT(stat, JOYSTICK_TYPE_UP))
        new_type = JOYSTICK_TYPE_UP;

    if (!CHECK_BIT(stat, JOYSTICK_TYPE_DOWN))
        new_type = JOYSTICK_TYPE_DOWN;

    if (!CHECK_BIT(stat, JOYSTICK_TYPE_LEFT))
        new_type = JOYSTICK_TYPE_LEFT;

    if (!CHECK_BIT(stat, JOYSTICK_TYPE_RIGHT))
        new_type = JOYSTICK_TYPE_RIGHT;

    if (!CHECK_BIT(stat, JOYSTICK_TYPE_CENTER))
        new_type = JOYSTICK_TYPE_CENTER;

    if ((old_type == new_type)) {
        return JOYSTICK_TYPE_NONE;
    }
    old_type = new_type;
    ;
    return new_type;
}

static void lcd_init()
{
    uint8_t res;
    res = ssd1351_basic_init();
    if (res != 0) {
        ssd1351_interface_debug_print("ssd1351: init failed.\n");
        return;
    }
    res = ssd1351_basic_clear();
    if (res != 0) {
        ssd1351_interface_debug_print("ssd1351: clear screen failed.\n");
        (void)ssd1351_basic_deinit();
        return;
    }
    return;
}

typedef struct _menu_tag {
    int x;
    int y;
    char *str;
    FP fp;
} MENU;

#define MENU_CNT       4
#define MENU_FONT_CLR  MAKE_RGB565(255, 255, 0)
#define MENU_FONT_BACK MAKE_RGB565(255, 255, 255)

static WAVE *pWavbg;
static void menu_volume();
static void menu_wav_play();
static void defalut_func();
extern void lis2dh12_read_data_polling();

static MENU mymemu[MENU_CNT] = {
    {10, 10, "adluna", defalut_func},
    {10, 30, "Sensor", lis2dh12_read_data_polling},
    {10, 50, "Wav Play", menu_wav_play},
    {10, 70, "Volume", menu_volume},
};

static void defalut_func()
{
    int y_base = 10;
    while (1) {
        fb_draw_rect(0, 0, 127, 127, MAKE_RGB565(51, 51, 255));
        fb_draw_string(4, y_base, "adLuna", MAKE_RGB565(255, 255, 51), SSD1351_FONT_12);
        fb_draw_string(4, y_base + 20, "Flash: 512K,1M,2M,4M", MAKE_RGB565(255, 255, 51), SSD1351_FONT_12);
        fb_draw_string(4, y_base + 40, "SRAM: 80KB", MAKE_RGB565(255, 255, 255), SSD1351_FONT_12);
        fb_draw_string(4, y_base + 60, "SoundMixer: 4 Ch", MAKE_RGB565(255, 0, 127), SSD1351_FONT_12);
        fb_update_lcd();
        while (1) {
            if (JOYSTICK_TYPE_NONE != get_joystick_type())
                return;
        }
    }
}

static int g_volume = 235;
static void menu_volume()
{
    char str_vol[64];
    fb_draw_rect(0, 0, 127, 127, MAKE_RGB565(255, 255, 255));
    sprintf(str_vol, "volume: %03d", g_volume);
    fb_draw_string(10, 50, str_vol, MAKE_RGB565(0, 0, 0), SSD1351_FONT_16);
    fb_update_lcd();
    while (1) {
        JOYSTICK_TYPE type = get_joystick_type();
        switch (type) {
            case JOYSTICK_TYPE_UP:
                if (g_volume < 255)
                    g_volume++;
                sound_vol(g_volume);
                break;

            case JOYSTICK_TYPE_DOWN:
                if (g_volume > 0)
                    g_volume--;
                sound_vol(g_volume);
                break;
            case JOYSTICK_TYPE_NONE:
                continue;
                break;
            default:
                return;
        }
        sprintf(str_vol, "volume: %03d", g_volume);
        fb_draw_rect(0, 50, 127, 70, MAKE_RGB565(255, 255, 255));
        fb_draw_string(10, 50, str_vol, MAKE_RGB565(0, 0, 0), SSD1351_FONT_16);
        fb_update_lcd();
    }
}

static void menu_wav_play()
{
    static bool is_playing = false;
    char str[64];
    fb_draw_rect(0, 0, 127, 127, MAKE_RGB565(255, 255, 255));
    sprintf(str, "Play: %s", is_playing ? "On" : "off");
    fb_draw_string(10, 50, str, MAKE_RGB565(0, 0, 0), SSD1351_FONT_16);
    fb_update_lcd();
    while (1) {
        JOYSTICK_TYPE type = get_joystick_type();
        switch (type) {
            case JOYSTICK_TYPE_UP:
                if (is_playing == false)
                    is_playing = true;
                amp_on(true);
                sound_play(pWavbg);

                break;

            case JOYSTICK_TYPE_DOWN:
                if (is_playing == true)
                    is_playing = false;
                sound_stop(pWavbg);
                amp_on(false);
                break;
            case JOYSTICK_TYPE_NONE:
                continue;
                break;
            default:
                return;
        }
        fb_draw_rect(0, 50, 127, 70, MAKE_RGB565(255, 255, 255));
        sprintf(str, "Play: %s", is_playing ? "On" : "off");
        fb_draw_string(10, 50, str, MAKE_RGB565(0, 0, 0), SSD1351_FONT_16);
        fb_update_lcd();
    }
}

int main()
{
    int i;
    evmboardinit();
    uart_config(DEBUG_CHANNEL, 115200, DATABITS_8, STOPBITS_1, UART_PARNONE);
    debugstring("================================================\r\n");
    debugprintf("adLuna Demo. System Clock(%dMhz)\r\n", get_ahb_clock() / 1000000);
    debugstring("================================================\r\n");

    sound_init();
    sound_vol(235); // sound vol, max = 255 (default)
    pWavbg        = sound_loadwavp((U8 *)bgwav_data, bgwav_end - bgwav_data);
    pWavbg->bLoop = TRUE;

    lcd_init();
    ssd1351_basic_rect(0, 0, 127, 127, MENU_FONT_BACK);

    fb_draw_bmp(0, 0, 127, 127, (uint8_t *)testbmp_data);
    fb_draw_rect(0, 10, 127, 30, MENU_FONT_BACK);
    for (i = 0; i < MENU_CNT; i++) {
        fb_draw_string(mymemu[i].x, mymemu[i].y, mymemu[i].str, MENU_FONT_CLR, SSD1351_FONT_16);
    }
    fb_update_lcd();
    // joystick
    //  GP8.1 ~ GP8.5
    //  8.5: Center
    //  8.4: right
    //  8.3: down
    //  8.2: left
    //  8.1: up
    *R_GPIDIR(8) = 0x1f << 1;

    int cur_secl_line = 0;
    while (1) {
        JOYSTICK_TYPE type = get_joystick_type();
        switch (type) {
            case JOYSTICK_TYPE_CENTER:
                mymemu[cur_secl_line].fp();
                break;
            case JOYSTICK_TYPE_RIGHT:
            case JOYSTICK_TYPE_LEFT:
                break;
            case JOYSTICK_TYPE_UP:
                if (cur_secl_line > 0)
                    cur_secl_line--;
                break;

            case JOYSTICK_TYPE_DOWN:
                if (cur_secl_line < (MENU_CNT - 1))
                    cur_secl_line++;
                break;
            default:
                continue;
        }
        if (type) {
            fb_draw_bmp(0, 0, 127, 127, (uint8_t *)testbmp_data);
            fb_draw_rect(0, 10 + (cur_secl_line * 20), 127, (cur_secl_line * 20) + 30, MENU_FONT_BACK);
            // draw menu
            for (i = 0; i < MENU_CNT; i++) {
                fb_draw_string(mymemu[i].x, mymemu[i].y, mymemu[i].str, MENU_FONT_CLR, SSD1351_FONT_16);
            }
            fb_update_lcd();
        }
    }

    while (1)
        ;
    return 0;
}
