#include "etc_driver/driver_ssd1351_basic.h"
#include "sdk.h"

extern void evmboardinit();
void        lcd_test()
{
    uint8_t res;
    res = ssd1351_basic_init();
    if(res != 0)
    {
        ssd1351_interface_debug_print("ssd1351: init failed.\n");

        return;
    }
    res = ssd1351_basic_clear();
    if(res != 0)
    {
        ssd1351_interface_debug_print("ssd1351: clear screen failed.\n");
        (void)ssd1351_basic_deinit();

        return;
    }
    ssd1351_basic_rect(0, 0, 127, 127, 0xffff);

    res = ssd1351_basic_string(0, 10, "Hello World!", 6, 0x0, SSD1351_FONT_16);
    res = ssd1351_basic_string(0, 40, "adLuna", 6, 0xF800, SSD1351_FONT_16);
    if(res != 0)
    {
        ssd1351_interface_debug_print("ssd1351: show string failed.\n");
        (void)ssd1351_basic_deinit();

        return;
    }
}

static void lcd_cmds(U8 cmd, U8 *buf, int len)
{
    ssd1351_interface_cmd_data_gpio_write(0);
    ssd1351_interface_spi_write(&cmd, 1);
    ssd1351_interface_cmd_data_gpio_write(1);
    ssd1351_interface_spi_write(buf, len);
}

#define SSD1351_CMD_SETCOLUMN      0x15 ///< See datasheet
#define SSD1351_CMD_SETROW         0x75 ///< See datasheet
#define SSD1351_CMD_WRITERAM       0x5C ///< See datasheet
#define SSD1351_CMD_READRAM        0x5D ///< Not currently used
#define SSD1351_CMD_SETREMAP       0xA0 ///< See datasheet
#define SSD1351_CMD_STARTLINE      0xA1 ///< See datasheet
#define SSD1351_CMD_DISPLAYOFFSET  0xA2 ///< See datasheet
#define SSD1351_CMD_DISPLAYALLOFF  0xA4 ///< Not currently used
#define SSD1351_CMD_DISPLAYALLON   0xA5 ///< Not currently used
#define SSD1351_CMD_NORMALDISPLAY  0xA6 ///< See datasheet
#define SSD1351_CMD_INVERTDISPLAY  0xA7 ///< See datasheet
#define SSD1351_CMD_FUNCTIONSELECT 0xAB ///< See datasheet
#define SSD1351_CMD_DISPLAYOFF     0xAE ///< See datasheet
#define SSD1351_CMD_DISPLAYON      0xAF ///< See datasheet
#define SSD1351_CMD_PRECHARGE      0xB1 ///< See datasheet
#define SSD1351_CMD_DISPLAYENHANCE 0xB2 ///< Not currently used
#define SSD1351_CMD_CLOCKDIV       0xB3 ///< See datasheet
#define SSD1351_CMD_SETVSL         0xB4 ///< See datasheet
#define SSD1351_CMD_SETGPIO        0xB5 ///< See datasheet
#define SSD1351_CMD_PRECHARGE2     0xB6 ///< See datasheet
#define SSD1351_CMD_SETGRAY        0xB8 ///< Not currently used
#define SSD1351_CMD_USELUT         0xB9 ///< Not currently used
#define SSD1351_CMD_PRECHARGELEVEL 0xBB ///< Not currently used
#define SSD1351_CMD_VCOMH          0xBE ///< See datasheet
#define SSD1351_CMD_CONTRASTABC    0xC1 ///< See datasheet
#define SSD1351_CMD_CONTRASTMASTER 0xC7 ///< See datasheet
#define SSD1351_CMD_MUXRATIO       0xCA ///< See datasheet
#define SSD1351_CMD_COMMANDLOCK    0xFD ///< See datasheet
#define SSD1351_CMD_HORIZSCROLL    0x96 ///< Not currently used
#define SSD1351_CMD_STOPSCROLL     0x9E ///< Not currently used
#define SSD1351_CMD_STARTSCROLL    0x9F ///< Not currently used

static const uint8_t initList[] = {SSD1351_CMD_COMMANDLOCK,
                                   1, // Set command lock, 1 arg
                                   0x12,
                                   SSD1351_CMD_COMMANDLOCK,
                                   1, // Set command lock, 1 arg
                                   0xB1,
                                   SSD1351_CMD_DISPLAYOFF,
                                   0, // Display off, no args
                                   SSD1351_CMD_CLOCKDIV,
                                   1,
                                   0xF1, // 7:4 = Oscillator Freq, 3:0 = CLK Div Ratio (A[3:0]+1 = 1..16)
                                   SSD1351_CMD_MUXRATIO,
                                   1,
                                   127,
                                   SSD1351_CMD_DISPLAYOFFSET,
                                   1,
                                   0x0,
                                   SSD1351_CMD_SETGPIO,
                                   1,
                                   0x00,
                                   SSD1351_CMD_FUNCTIONSELECT,
                                   1,
                                   0x01, // internal (diode drop)
                                   SSD1351_CMD_PRECHARGE,
                                   1,
                                   0x32,
                                   SSD1351_CMD_VCOMH,
                                   1,
                                   0x05,
                                   SSD1351_CMD_NORMALDISPLAY,
                                   0,
                                   SSD1351_CMD_CONTRASTABC,
                                   3,
                                   0xC8,
                                   0x80,
                                   0xC8,
                                   SSD1351_CMD_CONTRASTMASTER,
                                   1,
                                   0x0F,
                                   SSD1351_CMD_SETVSL,
                                   3,
                                   0xA0,
                                   0xB5,
                                   0x55,
                                   SSD1351_CMD_PRECHARGE2,
                                   1,
                                   0x01,
                                   SSD1351_CMD_DISPLAYON,
                                   0,  // Main screen turn on
                                   0}; // END OF COMMAND LIST

void mylcd_init()
{
    const uint8_t *addr = (const uint8_t *)initList;
    uint8_t        cmd, x, numArgs;

    while((cmd = *(addr++)) > 0)
    { // '0' command ends list
        x       = *(addr++);
        numArgs = x & 0x7F;
        if(cmd != 0xFF)
        { // '255' is ignored
            lcd_cmds(cmd, (U8*)addr, numArgs);
        }
        addr += numArgs;
    }
}
int main()
{
    evmboardinit();
    uart_config(DEBUG_CHANNEL, 115200, DATABITS_8, STOPBITS_1, UART_PARNONE);
    debugstring("================================================\r\n");
    debugprintf("adLuna LCD. System Clock(%dMhz)\r\n", get_ahb_clock() / 1000000);
    debugstring("================================================\r\n");
    lcd_test();
    mylcd_init();
    ssd1351_basic_rect(0, 0, 127, 127, 0xffff);
    ssd1351_basic_rect(0, 50, 16, 66, 0xf800);

    while(1)
        ;
    return 0;
}
