
#include "sdk.h"

extern const U8 USB_Device_Descriptor[];
extern const U8 USB_Configuration_Descriptor[];

extern void evmboardinit();

int WaitUSBBulkData(int timeout)
{
    BYTE ep_irq;
    BYTE usb_irq;
    int  i  = 0;
    int  re = -1;
    while(1)
    {
        ep_irq  = *(volatile U8 *)(__USBEI_REG);
        usb_irq = *(volatile U8 *)(__USBUI_REG);

        if(ep_irq & 1) // EP0_IRQ
        {
            ep0_isr();
        }
        if(ep_irq & (1 << 1)) // EP1_IRQ
        {
            re = 1;
            break;
        }

        if(usb_irq & USB_RESET_IRQ)
        {
            usb_reset();
        }
        else if(usb_irq & USB_RESUME_IRQ)
        {
            usb_resume();
        }
        else if(usb_irq & USB_SUSPEND_IRQ)
        {
            usb_suspend();
        }
        if(timeout != -1)
            i++;
        if(i > timeout)
            break;
        *(volatile U8 *)__USBEI_REG = ep_irq;  // Clear Interrupt
        *(volatile U8 *)__USBUI_REG = usb_irq; // Clear Interrupt
    }
    *(volatile U8 *)__USBEI_REG = ep_irq;  // Clear Interrupt
    *(volatile U8 *)__USBUI_REG = usb_irq; // Clear Interrupt

    return re;
}

int main()
{
    evmboardinit();

    uart_config(DEBUG_CHANNEL, 115200, DATABITS_8, STOPBITS_1, UART_PARNONE);
    debugstring("START\r\n");
    debugstring("================================================\r\n");
    debugprintf("adLuna USB Bulk Transfer.System Clock(%dMhz)\r\n", get_ahb_clock() / 1000000);
    debugstring("================================================\r\n");

    usb_clock_init();
    usb_set_description((BYTE *)USB_Configuration_Descriptor, (BYTE *)USB_Device_Descriptor);
    usb_init(USB_BULK_MODE);

    int  ret;
    char buf[512];
    while(1)
    {
        debugstring("Waiting for data....\r\n");
        while(WaitUSBBulkData(-1) == -1)
            ;
        ret = read_usb(buf);
        debugprintf("%d byte received\r\n", ret);
        write_usb(buf, ret);
        // for debug
        buf[ret] = 0;
        debugstring(buf);
        debugstring("\r\n");
    }

    return 0;
}
