/*****************************************************************************
 *
 * Copyright (C) 2018      Advanced Digital Chips, Inc. All Rights Reserved.
 *						http://www.adc.co.kr
 *
 * THIS SOFTWARE IS PROVIDED BY ADCHIPS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ADCHIPS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 *****************************************************************************/

#include "sdk.h"

U32 get_pll()
{
    int n;
    int r;
    int od;
    int val;
    val = *R_SYSCON_PLLCON;
    n   = val & 0x1ff;
    r   = (val >> 16) & 0x3f;
    od  = (val >> 24) & 3;
    if(od == 2)
        od = 2;
    else
        od += 1;
    return (OSC_CLOCK * n) / (r * od);
}

U32 get_ahb_clock()
{
    if((*R_SYSCON_CLKCON) & 1) // PLL?
    {
        return get_pll() / (((*R_SYSCON_HCLKDIV) & 0xf) + 1);
    }
    else
    {
        return OSC_CLOCK;
    }
}

U32 get_apb_clock()
{
    return get_ahb_clock() / 2;
}

U32 get_clk16()
{
    U32 div    = *R_SYSCON_CLK16DIV;
    U32 source = 0;

    div &= 0xf;
    switch(((*R_SYSCON_CLKDCON) >> 18) & 3)
    {
    case 0:
        source = OSC_CLOCK;
        break;
    case 1:
        source = get_pll();
        break;
    default:
        return 0;
    }
    return source / (div + 1);
}

U32 get_clk1_5()
{
    U32 source;
    switch(((*R_SYSCON_CLKDCON) >> 8) & 3)
    {
    case 0:
        source = OSC_CLOCK;
        break;
    case 1:
        source = get_pll();
        break;
    default:
        return 0;
    }
    return ((source * 10) / 15);
}

U32 get_clk2_5()
{
    U32 source;
    switch(((*R_SYSCON_CLKDCON) >> 10) & 3)
    {
    case 0:
        source = OSC_CLOCK;
        break;
    case 1:
        source = get_pll();
        break;
    default:
        return 0;
    }
    return ((source * 10) / 25);
}

U32 get_clk4_5()
{
    U32 source;
    switch(((*R_SYSCON_CLKDCON) >> 12) & 3)
    {
    case 0:
        source = OSC_CLOCK;
        break;
    case 1:
        source = get_pll();
        break;
    default:
        return 0;
    }
    return ((source * 10) / 25);
}

U32 get_clk5()
{
    U32 source;
    switch(((*R_SYSCON_CLKDCON) >> 14) & 3)
    {
    case 0:
        source = OSC_CLOCK;
        break;
    case 1:
        source = get_pll();
        break;
    default:
        return 0;
    }
    return (source / 5);
}

U32 get_clk32()
{
    U32 source;
    switch(((*R_SYSCON_CLKDCON) >> 16) & 3)
    {
    case 0:
        source = OSC_CLOCK;
        break;
    case 1:
        source = get_pll();
        break;
    default:
        return 0;
    }
    return source / 32;
}
static U32 get_source_clock(U32 bitval)
{
    U32 clk = 0;
    int n   = (bitval >> 4) & 0x7;
    switch(n)
    {
    case 0:
        clk = OSC_CLOCK;
        break;
    case 1:
        clk = get_clk16();
        break;
    case 2:
        clk = get_clk1_5();
        break;
    case 3:
        clk = get_clk2_5();
        break;
    case 4:
        clk = get_clk4_5();
        break;
    case 5:
        clk = get_clk5();
        break;
    case 6:
        clk = get_clk32();
        break;
    default:
        return 0;
    }
    int div = bitval & 0xf;
    return clk / (div + 1);
}

U32 get_clock(DEV_CLOCK_TYPE devnum)
{
    U32 *ctrl_reg = (U32 *)(R_SYSCON_PERI0 + (devnum >> 2));
    U32  regval   = (*ctrl_reg) >> ((devnum & 0x3) * 8);
    return get_source_clock(regval & 0xff);
}
