#include "sdk.h"

#define USB_BUF_SIZE 512

typedef struct _USB_RING_BUF
{
    U16 uRingBufRxCtr;            // Number of characters in the Rx ring buffer
    U8 *uRingBufRxInPtr;          // Pointer to where next character will be inserted
    U8 *uRingBufRxOutPtr;         // Pointer from where next character will be extracted
    U8  uRingBufRx[USB_BUF_SIZE]; // Ring buffer character storage (Rx)
} USB_RING_BUF;

USB_RING_BUF usbbuf;

void InitRingBuf_usb()
{
    usbbuf.uRingBufRxCtr    = 0;
    usbbuf.uRingBufRxInPtr  = &usbbuf.uRingBufRx[0];
    usbbuf.uRingBufRxOutPtr = &usbbuf.uRingBufRx[0];
}

U32 total_in_data_len;

void usb_virtual_com_isr(void)
{
    BYTE          usb_data[64];
    volatile U32  len;
    volatile U32  k    = 0;
    USB_RING_BUF *pbuf = &usbbuf;
    BYTE          ep_irq;
    BYTE          usb_irq;

    ep_irq  = *(volatile U8 *)(__USBEI_REG);
    usb_irq = *(volatile U8 *)(__USBUI_REG);

    if(ep_irq & 1) // EP0_IRQ
    {
        ep0_isr();
    }
    if(ep_irq & (1 << 1)) // EP1_IRQ
    {
        k   = 0;
        len = read_usb(usb_data);
        while(len)
        {
            if(pbuf->uRingBufRxCtr < USB_BUF_SIZE)
            {
                pbuf->uRingBufRxCtr++;
                *pbuf->uRingBufRxInPtr = usb_data[k++];
                pbuf->uRingBufRxInPtr++;
                if(pbuf->uRingBufRxInPtr == &pbuf->uRingBufRx[USB_BUF_SIZE])
                {
                    pbuf->uRingBufRxInPtr = &pbuf->uRingBufRx[0];
                }
            }
            len--;
        }
    }

    if(usb_irq & USB_RESET_IRQ)
    {
        usb_reset();
    }
    else if(usb_irq & USB_RESUME_IRQ)
    {
        usb_resume();
    }
    else if(usb_irq & USB_SUSPEND_IRQ)
    {
        usb_suspend();
    }
    *(volatile U8 *)__USBEI_REG = ep_irq;  // Clear Interrupt
    *(volatile U8 *)__USBUI_REG = usb_irq; // Clear Interrupt
}

int UsbGetCh(U8 *buf)
{
    int i = 0;

    USB_RING_BUF *pbuf = &usbbuf;

    CRITICAL_ENTER();
    if(pbuf->uRingBufRxCtr > 0)
    {                                       /* No, decrement character count */
        pbuf->uRingBufRxCtr--;              /* No, decrement character count */
        buf[0] = *pbuf->uRingBufRxOutPtr++; /* Get character from buffer */
        /* Wrap OUT pointer     */
        if(pbuf->uRingBufRxOutPtr == &pbuf->uRingBufRx[USB_BUF_SIZE])
        {
            pbuf->uRingBufRxOutPtr = &pbuf->uRingBufRx[0];
        }
        CRITICAL_EXIT();
        i = 1;
    }
    else
    {
        CRITICAL_EXIT();
        i = 0;
    }
    return i;
}

void UsbUartPutCh(U8 data)
{
    write_usb(&data, 1);
}

void UsbUartPuts(U8 *data)
{
    int siz = strlen(data);
    int len = 0;

    while(siz > 0)
    {
        if(siz > 64)
            len = 64;
        else
            len = siz;
        write_usb(data, len);
        data += len;
        siz -= len;
    }
}
