/*-----------------------------------------------------------------------*/
/* Low level disk I/O module SKELETON for FatFs     (C)ChaN, 2019        */
/*-----------------------------------------------------------------------*/
/* If a working storage control module is available, it should be        */
/* attached to the FatFs via a glue function rather than modifying it.   */
/* This is an example of glue functions to attach various exsisting      */
/* storage control modules to the FatFs module with a defined API.       */
/*-----------------------------------------------------------------------*/

#include "sdk.h"			/* Obtains integer types */
#include "fatfs/diskio.h"		/* Declarations of disk functions */
#include "fatfs/serialflash_fs.h"


DSTATUS disk_initialize(BYTE drv)
{
	PRINTLINE;
	if(flash_fs_init())
		return RES_OK;
	
	return RES_ERROR;
}


/*-----------------------------------------------------------------------*/
/* Return Disk Status                                                    */

DSTATUS disk_status(
	BYTE drv		/* Physical drive nmuber (0..) */
)
{
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */

DRESULT disk_read(
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE* buff,		/* Data buffer to store read data */
	LBA_t sector,	/* Sector address (LBA) */
	UINT count		/* Number of sectors to read (1..255) */
)
{
	return flash_fs_readsects(sector,buff,count);
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */
/*-----------------------------------------------------------------------*/

#if FF_FS_READONLY == 0

DRESULT disk_write (
	BYTE drv,			/* Physical drive nmuber to identify the drive */
	const BYTE *buff,	/* Data to be written */
	LBA_t sector,		/* Start sector in LBA */
	UINT count			/* Number of sectors to write */
)
{

	return flash_fs_updatesects(sector,(void*)buff,count);
}

#endif


/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */

DRESULT disk_ioctl(
	BYTE drv,		/* Physical drive nmuber (0) */
	BYTE ctrl,		/* Control code */
	void* buff		/* Buffer to send/receive data block */
)
{
	DRESULT res = RES_PARERR;

	switch (ctrl) {
	case CTRL_SYNC:
		flash_fs_flushdata();
		res = RES_OK;
		break;

	case GET_SECTOR_COUNT:
		*(LBA_t*)buff = flash_fs_sector_count();
		break;

	case GET_SECTOR_SIZE:
		*(WORD*)buff = flash_fs_sector_size();
		res = RES_OK;
		break;
	case GET_BLOCK_SIZE:
		*(DWORD*)buff = 1;
		res = RES_OK;
		break;
	}

	return res;
}

/*
Currnet time is returned with packed into a DWORD value. The bit field is as follows:

bit31:25
Year from 1980 (0..127)
bit24:21
Month (1..12)
bit20:16
Day in month(1..31)
bit15:11
Hour (0..23)
bit10:5
Minute (0..59)
bit4:0
Second / 2 (0..29)
*/
#if FF_FS_NORTC == 0
DWORD get_fattime()
{
	return ((DWORD)(FF_NORTC_YEAR - 1980) << 25 | (DWORD)FF_NORTC_MON << 21 | (DWORD)FF_NORTC_MDAY << 16);
}
#endif
