/*------------------------------------------------------------------------*/
/* A Sample Code of User Provided OS Dependent Functions for FatFs        */
/*------------------------------------------------------------------------*/

#include "sdk.h"
#include "fatfs/ff.h"
#include "fatfs/ffconf.h"

static U32 fs_int_disable = 0;

/*
if you use file-io function at ISR, you should register interrupt-number
*/
void fs_enable_support_interrupt(INTERRUPT_TYPE intnum, BOOL b)
{
	if (b)
		fs_int_disable |= 1 << intnum;
	else
		fs_int_disable&= ~(1 << intnum);

}

#if FF_USE_LFN == 3	/* Use dynamic memory allocation */

/*------------------------------------------------------------------------*/
/* Allocate/Free a Memory Block                                           */
/*------------------------------------------------------------------------*/

#include <stdlib.h>		/* with POSIX API */


void* ff_memalloc (	/* Returns pointer to the allocated memory block (null if not enough core) */
	UINT msize		/* Number of bytes to allocate */
)
{
	return malloc((size_t)msize);	/* Allocate a new memory block */
}


void ff_memfree (
	void* mblock	/* Pointer to the memory block to free (no effect if null) */
)
{
	free(mblock);	/* Free the memory block */
}

#endif


#if FF_FS_REENTRANT	/* Mutal exclusion */

int ff_mutex_create (int vol) __attribute__((weak));		/* Create a sync object */
void ff_mutex_delete (int vol) __attribute__((weak));		/* Delete a sync object */
int ff_mutex_take (int vol) __attribute__((weak));		/* Lock sync object */
void ff_mutex_give (int vol) __attribute__((weak));		/* Unlock sync object */



static U32 org_int_mask;

/*------------------------------------------------------------------------*/
/* Create a Mutex                                                         */
/*------------------------------------------------------------------------*/
/* This function is called in f_mount function to create a new mutex
/  or semaphore for the volume. When a 0 is returned, the f_mount function
/  fails with FR_INT_ERR.
*/

int ff_mutex_create (	/* Returns 1:Function succeeded or 0:Could not create the mutex */
	int vol				/* Mutex ID: Volume mutex (0 to FF_VOLUMES - 1) or system mutex (FF_VOLUMES) */
)
{
	
	return 1;
}


/*------------------------------------------------------------------------*/
/* Delete a Mutex                                                         */
/*------------------------------------------------------------------------*/
/* This function is called in f_mount function to delete a mutex or
/  semaphore of the volume created with ff_mutex_create function.
*/

void ff_mutex_delete (	/* Returns 1:Function succeeded or 0:Could not delete due to an error */
	int vol				/* Mutex ID: Volume mutex (0 to FF_VOLUMES - 1) or system mutex (FF_VOLUMES) */
)
{
}


/*------------------------------------------------------------------------*/
/* Request a Grant to Access the Volume                                   */
/*------------------------------------------------------------------------*/
/* This function is called on enter file functions to lock the volume.
/  When a 0 is returned, the file function fails with FR_TIMEOUT.
*/

int ff_mutex_take (	/* Returns 1:Succeeded or 0:Timeout */
	int vol			/* Mutex ID: Volume mutex (0 to FF_VOLUMES - 1) or system mutex (FF_VOLUMES) */
)
{
	/*register Interrupt Number what used filesystem
	DMA(Sound) ISR is use FS
	*/
	CRITICAL_ENTER();
	org_int_mask = *R_INTMASK;
	*R_INTMASKCLR = fs_int_disable;
	CRITICAL_EXIT();
	return 1;

}



/*------------------------------------------------------------------------*/
/* Release a Grant to Access the Volume                                   */
/*------------------------------------------------------------------------*/
/* This function is called on leave file functions to unlock the volume.
*/

void ff_mutex_give (
	int vol			/* Mutex ID: Volume mutex (0 to FF_VOLUMES - 1) or system mutex (FF_VOLUMES) */
)
{
	*R_INTMASK = org_int_mask;

}

#endif	/* FF_FS_REENTRANT */


